local ourTables = require("Bashmatic.ftext").TableMaker

Bashmatic = Bashmatic or {
    ["commands"] = {
        start = nil,
        before = nil,
        during = nil,
        duringTime = nil,
        after = nil
    },
    ["death"] = {
        phrase = nil,
        event = nil
    },
    ["enemies"] = {
        group = false,
        targets = {}
    },
    ["enabled"] = false,
    ["bashing"] = false,
    ["hunting"] = {
        events = {}
    }
}

function handleInstall(_, name)
    if name:lower() == "Bashmatic" then
        cecho("<MediumPurple>Bashmatic: Welcome to Bashmatic. See bm help to get started.")
    end
end
registerAnonymousEventHandler("sysInstallPackage", handleInstall)

function handleShutdown()

    if Bashmatic.death.event ~= nil then killTrigger(Bashmatic.death.event) end
    killBashingTriggers()
    bmDisable()
    toggleBashing(false)
    Bashmatic.death.event = nil

    table.save(getMudletHomeDir() .. "/Bashmatic.lua", Bashmatic)

end
registerAnonymousEventHandler("sysExitEvent", handleShutdown)

function handleLoadEvent()

    if io.exists(getMudletHomeDir() .. "/Bashmatic.lua") then
        table.load(getMudletHomeDir() .. "/Bashmatic.lua", Bashmatic)
    end

    if Bashmatic.death.phrase ~= nil then
        Bashmatic.death.event = tempTrigger(Bashmatic.death.phrase,handleMobDeath)
    end

    cecho("<DarkSlateBlue>Bashmatic: Loaded settings. Let's bash some mobs.")

end
registerAnonymousEventHandler("sysLoadEvent", handleLoadEvent)

function bmDisable()
    Bashmatic.enabled = false
end

function toggleBashing(status)
    Bashmatic.bashing = status
end

function handleMobDeath()
    if Bashmatic.commands.after then
        send(Bashmatic.commands.after)
    end
    toggleBashing(false)
    if Bashmatic.enabled then
        if table.size(Bashmatic.hunting.events) > 0 then
            -- active triggers
            send("look")
        else
            -- recreate triggers and then look
            recreateTriggers()
            send("look")
        end
    end
end

function handleDuringTick()
    send(Bashmatic.commands.during)
    if Bashmatic.bashing then tempTimer(Bashmatic.commands.duringTime,handleDuringTick) end
end

-- Functions for handling inline matching

function handleBashingMatches(match)
    if not Bashmatic.enabled or Bashmatic.bashing then
        -- bashing is disabled or we're currently bashing something, so clear the triggers
        killBashingTriggers()
    else
        if Bashmatic.commands.before then send(Bashmatic.commands.before) end
        send(Bashmatic.commands.start .. match)
        if Bashmatic.commands.during then tempTimer(Bashmatic.commands.duringTime,handleDuringTick) end
    end
end

function bmCreateTrigger(keyword)
    table.insert(Bashmatic.hunting.events, tempTrigger(keyword, handleBashingMatches(keyword), 1))
end

-- Functions for handling adding targets

function inlineAddTarget(target, area)
    if area == nil then 
        cecho("<DarkSlateBlue>Bashmatic: Added " .. target .. " to hunting targets.<reset>") 
        if not table.contains(Bashmatic.enemies.targets, target) then
            table.insert(Bashmatic.enemies.targets, target)
        end
    end

    if area ~= nil then 
        cecho("<DarkSlateBlue>Bashmatic: Added " .. target .. " to hunting targets in " .. area .. ".<reset>")
        if table.contains(Bashmatic.enemies.targets, area) then
            table.insert(Bashmatic.enemies.targets.area, target)
        else
            Bashmatic.enemies.targets.area = {}
            table.insert(Bashmatic.enemies.targets.area, target)
        end
    end
    
    if not Bashmatic.bashing and Bashmatic.enabled then
        bmCreateTrigger(target)
    end
end

function recreateTriggers()
    if Bashmatic.enemies.group then
        -- Grouping enemies by areas
        local thisArea = GetRoomAreaName(GetRoomArea(GetPlayerRoom())) or nil
        if thisArea == nil then
            cecho("<firebrick>Bashmatic: We attempted to locate your area but failed.\nGot: " .. thisArea .. "<reset>")
            return
        end
        if not table.contains(Bashmatic.enemies.targets, thisArea) then
            cecho("<firebrick>Bashmatic: You haven't added any enemies for the area " .. thisArea .. "<reset>")
            return
        end
        for _, keyword in ipairs(Bashmatic.enemies.targets.thisArea) do
            bmCreateTrigger(keyword)
        end
    else
        -- Not grouping enemies by areas
        for _, keyword in ipairs(Bashmatic.enemies.targets) do
            bmCreateTrigger(keyword)
        end
    end
end

function killBashingTriggers()
    if table.size(Bashmatic.hunting.events) > 0 then
        for _, tid in ipairs(Bashmatic.hunting.events) do
            killTrigger(tid)
        end
    end
    Bashmatic.hunting.events = {}
end

function displayTargetList(start, endi, page, pages)
    local thisTable = ourTables:new({
        title = "Bashmatic Target List",
        printTitle = true,
        titleColor = "<MediumPurple>",
        printHeaders = false,
        separateRows = false
    })
    thisTable:addColumn({
        name = "Index",
        width = 8,
        textColor = "<firebrick>"
    })
    thisTable:addColumn({
        name = "Target",
        width = 72,
        textColor = "<Slategrey>"
    })
    for i=start,i<=endi,1 do
        thisTable:addRow({
            i,
            Bashmatic.enemies.targets[i]
        })
    end
    cecho(thisTable:assemble())
    if page ~= nil then
        cecho("<DarkSlateBlue>Displaying " .. start .. " to " .. endi .. " (Page " .. page .. " of " .. pages .. ")<reset>")
    end
end

function displayAreaTargetList(area, start, endi, page, pages)
    local thisTable = ourTables:new({
        title = "Bashmatic Target List",
        printTitle = true,
        titleColor = "<MediumPurple>",
        printHeaders = false,
        separateRows = false
    })
    thisTable:addColumn({
        name = "Index",
        width = 8,
        textColor = "<firebrick>"
    })
    thisTable:addColumn({
        name = "Target",
        width = 72,
        textColor = "<Slategrey>"
    })
    for i=start,i<=endi,1 do
        thisTable:addRow({
            i,
            Bashmatic.enemies.targets.area[i]
        })
    end
    cecho("<MediumPurple>Target Listing for area " .. area .. "<reset>")
    if page ~= nil then
        cecho("<DarkSlateBlue>Displaying " .. start .. " to " .. endi .. " (Page " .. page .. " of " .. pages .. ")<reset>")
    end
end

function squashTargetGroups()
    local finalTable = {}
    for _, area in ipairs(Bashmatic.enemies.targets) do
        if table.size(Bashmatic.enemies.targets.area) > 0 then
            for _, enemy in ipairs(Bashmatic.enemies.targets.area) do
                table.insert(finalTable,enemy)
            end
        end
    end
    Bashmatic.enemies.targets = finalTable
end